# Called by Xcode from a Run Script build phase in the Typed target.

set -x

export CURRENT_PROJECT_VERSION

function ProcessInfoPListWithEnvironment
{
  echo "Source Info.plist file is at ""$SRCROOT"/"$INFOPLIST_FILE"
  echo "Target Info.plist file is at ""${TARGET_BUILD_DIR}/${INFOPLIST_PATH}"
  echo "Revision Number is $RM_BUNDLE_BUILD_VERSION"

  perl -pe 's/\$\{([^}]*)\}/$ENV{$1}/g' < "$SRCROOT"/"$INFOPLIST_FILE" > "${TARGET_BUILD_DIR}/${INFOPLIST_PATH}"

  dsym_info_plist="$BUILT_PRODUCTS_DIR/$PRODUCT_NAME.app.dSYM/Contents/Info.plist"

  if [ -f "$dsym_info_plist" ]
  then
    perl -pe 's/\$\{([^}]*)\}/$ENV{$1}/g' < "$SRCROOT"/"$INFOPLIST_FILE" > "$dsym_info_plist"
  fi
}

# first, let's try git
git_path=($( which git ))
if [ -x "$git_path" ]
then
  branch=${1:-'master'}
  git_status=($( $git_path status 2>&1))
  if [[ $git_status != *"fatal"* ]]
  then
    # work out the current branch
    branch_name=$(git symbolic-ref -q HEAD)
    branch_name=${branch_name##refs/heads/}
    branch=${branch_name:-HEAD}

    # get the number of commits on this branch
    buildNumber=$(expr $($git_path rev-list $branch --count) - $($git_path rev-list HEAD..$branch --count))

	# append a 'b' if doing a beta build
	configuration=${CONFIGURATION}
	if [[ $configuration == *"Beta"* ]]
	then
		buildNumber="${buildNumber}b"
	fi

    export RM_BUNDLE_BUILD_VERSION="$buildNumber"
	ProcessInfoPListWithEnvironment

  exit 0
  fi
fi
