#!/usr/bin/env ruby

require 'optparse'

libraryDirectoryPath = File.dirname(__FILE__)
$LOAD_PATH << libraryDirectoryPath

require 'bundle.rb'
require 'codesign.rb'

options = {}

opt_parser = OptionParser.new do |opt|
	opt.banner = "Usage: codesign_childbundles.rb [OPTIONS]"
	opt.separator  ""
	opt.separator  "Options"
	
	opt.on("-a", "--applicationPath [APP_PATH]", "the path to the application") do |application_path|
    	options[:applicationPath] = application_path
	end
	
	opt.on("-c", "--codesignIdentity [CODESIGN_IDENTITY]", "the codesign identity") do |codesign_identity|
    	options[:codesignIdentity] = codesign_identity
	end
end

opt_parser.parse!

application_path = options[:applicationPath]
codesign_identity = options[:codesignIdentity]

if !application_path || !codesign_identity
	puts opt_parser
	exit -1
end

def log(string = "")
	puts string
	$stdout.flush
end

def logError(error = "")
	log "error: #{error}"
end

log "codesign child bundles with identity \"#{codesign_identity}\""

# Collect bundles and codesign them

codesignSucccess = true

childBundleLocations = Bundle.childBundleLocationsOfBundleAtLocation application_path

# childBundleLocationsOfBundleAtLocation is a breadth first search, by reversing the list we get the deepest components -> shallowest components
# codesign on 10.9 complains at codesign time if a child bundle isn't codesigned, this ensures when signing frameworks that contain frameworks, the deepest frameworks are signed first
childBundleLocations.reverse_each do |currentBundleLocation|
# 	currentBundleExecutableArchitectures = CFBundleCopyExecutableArchitecturesForURL currentBundleLocation
# 	next if currentBundleExecutableArchitectures.nil?
	
	codesign, codesignError = Codesign.signBundleAtLocationUsingIdentityAndPreserveAllMetadata currentBundleLocation, codesign_identity
	
	codesignSucccess = codesignSucccess && codesign
	
	unless codesign
		logError "couldn't codesign bundle at location #{currentBundleLocation}, #{codesignError}"
		next
	end
	
	next
end

unless codesignSucccess
    exit 1
end
