
function svn_revision {
	location="${1}"
	revision=$(svn info "${location}" | grep '^Revision:' | cut -d' ' -f2)
	echo "${revision}"
}

function repository_url {
	workingcopy_path="${1}"
	repository_url=$(svn info "${workingcopy_path}" | grep '^URL:' | cut -d' ' -f2)
	echo "${repository_url}"
}

function repository_diff {
	workingcopy_path="${1}"
	workingcopy_revision="${2}"
	if [ -z "${workingcopy_revision}" ]
	then
		workingcopy_revision=$(svn_revision "${workingcopy_path}")
	fi
	
	repository_url=$(repository_url "${workingcopy_path}")
	repository_revision=$(svn_revision "${repository_url}")

	if [ "${workingcopy_revision}" -eq "${repository_revision}" ]
	then
		echo "> skipping diff ${workingcopy_path}; working copy revision is equal to the server revision" >&2
		return
	fi
	
	echo "> diff ${workingcopy_path} from: ${workingcopy_revision} to: ${repository_revision}" >&2
	
	revision_range="${workingcopy_revision}:${repository_revision}"
	
	diff_summary=$(svn diff --summarize --revision "${revision_range}" "${repository_url}")
	if [ -z "${diff_summary}" ]
	then
		echo "> skipping diff ${workingcopy_path}; no changes to this folder on the server in the revision range ${revision_range}" >&2
		return
	fi
	
	echo "> there are changes on the server to approve in the revision range ${revision_range}" >&2
	
	svn diff --force --revision "${revision_range}" "${workingcopy_path}" &>/dev/null
	
	echo "${repository_revision}"
}

function externals_diff {
	externals_directory="${1%/}"
	if [ -z "${externals_directory}" ]
	then
		externals_directory="Externals"
	fi
	IFS=$'\r\n' externals_definition=($( svn propget svn:externals "${externals_directory}" ))
	
	if [ -z "${externals_definition}" ]
	then
		echo -e "> svn:externals is empty, aborting"
		return
	fi
	
	new_externals_definition=( "${externals_definition[@]}" )
	next_externals_index=0
	
	for current_external_definition in "${externals_definition[@]}"
	do
		current_externals_index=$next_externals_index
		next_externals_index=$(( next_externals_index + 1 ))
		
		[ $current_externals_index -gt 0 ] && echo -e ""
		echo -e "Diff external\t\"${current_external_definition}\"" >&2
		
		IFS=$' ' current_external_fields=($( echo "${current_external_definition}" ))
		
		current_external_field_count=${#current_external_fields[@]}
		if [ ! $current_external_field_count -eq 3 ]
		then
			echo -e "> skipping diff; this external isn't frozen to a specific revision" >&2
			continue
		fi
		
		current_external_path="${externals_directory}/${current_external_fields[0]}"
		current_external_revision="${current_external_fields[1]#-r}"
		
		compared_revision=$(repository_diff "${current_external_path}" "${current_external_revision}")
		[ -z "${compared_revision}" ] && continue
		
		while [ 1 ]
		do
			read -r -p "> merge changes up to revision ${compared_revision} [y/n]: "
			if [ "${REPLY}" != "y" -a "${REPLY}" != "n" ]
			then
				echo "> unknown option ${REPLY}; try again" >&2
				continue
			fi
			
			break
		done

		if [ "${REPLY}" = "n" ]
		then
			echo -e "> user intervention, skipping merge"
			continue
		fi
		
		svn up --revision "${compared_revision}" "${current_external_path}"
		
		current_external_fields[1]="-r${compared_revision}"
		
		new_current_external_definition="${current_external_fields[*]}"
		new_externals_definition[$current_externals_index]=$new_current_external_definition
	done
	
	reconstructed_externals_definition=""
	for current_external_definition in "${new_externals_definition[@]}"
	do
		reconstructed_externals_definition="${reconstructed_externals_definition}${current_external_definition}\n"
	done
	
	printf "\n" >&2
	
	# Evaluate the newline characters from the reconstruction and keep the output in a single string
	svn propset svn:externals "$( echo -e ${reconstructed_externals_definition} )" "${externals_directory}"
	
	svn commit --message "Update external revisions" "${externals_directory}"
}

function externals_edit {
	externals_directory="${1}"
	if [ -z "${externals_directory}" ]
	then
		externals_directory="Externals"
	fi
	
	svn propedit svn:externals "${externals_directory}"
}

function branch_tag {
	tag_name="${1}"
	if [ -z "${tag_name}" ]
	then
		echo -e "tag name must be given" >&2
		return
	fi
	
	workingcopy_path="${2}"
	if [ -z "${workingcopy_path}" ]
	then
		workingcopy_path="${PWD}"
	fi
	
	workingcopy_revision=$(svn_revision "${workingcopy_path}")
	
	repository_url=$(repository_url "${workingcopy_path}")
	repository_tags_url=$(echo "${repository_url}" | sed -E 's/(trunk$|branches\/[^\/][^\/]*$)/tags/')
	
	message="Tag ${tag_name}"
	from_branch_url="${repository_url}@${workingcopy_revision}"
	new_tag_url="${repository_tags_url}/${tag_name}"
	
	svn cp --message "${message}" "${from_branch_url}" "${new_tag_url}"
}

function lowercase {
	tr '[:upper:]' '[:lower:]'
}

function strip_whitespace {
	tr -d ' '
}
