#!/System/Library/Frameworks/Ruby.framework/Versions/1.8/usr/bin/ruby

require 'osx/cocoa'
include OSX

def usage
	print "Usage:\n"
	print "-lprojPath: path to the development region lproj"
	print "\n"
	exit 1
end

if ARGV.count == 0
	usage
end

def finish
	print "============  XIB STRINGS EXTRACTION SUCCESS   ============\n"
end

def fail(reason = nil, severity = :warning)
	print "\n"
	print "============  XIB STRINGS EXTRACTION FAILURE   ============\n"
	unless reason.nil?
		print "error: #{reason}\n"
	end
	print "\n"
	
	if severity == :fatal
		exit 1
	end
	
	def finish
		fail nil, :fatal
	end
end

# Check arguments

lprojPath = NSUserDefaults.standardUserDefaults.stringForKey("lprojPath")
if lprojPath.nil?
	fail "the lproj path must be specified using -lprojPath", :fatal
end
lprojLocation = NSURL.fileURLWithPath lprojPath

ibtoolLocation = `xcrun -find ibtool`.to_ns.stringByTrimmingCharactersInSet_(NSCharacterSet.whitespaceAndNewlineCharacterSet)

lprojDirectoryEnumerator = NSFileManager.defaultManager.enumeratorAtPath lprojLocation.path
while currentFileName = lprojDirectoryEnumerator.nextObject do
	currentLocation = lprojLocation.URLByAppendingPathComponent currentFileName
	
	next unless currentLocation.pathExtension.isEqual_ "xib"
	
	exportLocation = lprojLocation.URLByAppendingPathComponent currentFileName.stringByDeletingPathExtension.stringByAppendingPathExtension("strings")
	
	ibtoolArguments = NSMutableArray.array
	ibtoolArguments << "--export-strings-file"
	ibtoolArguments << exportLocation.path
	ibtoolArguments << currentLocation.path
	
	$stdout.flush
	exportStringsTask = NSTask.launchedTaskWithLaunchPath_arguments_ ibtoolLocation, ibtoolArguments
	exportStringsTask.waitUntilExit
	
	unless exportStringsTask.terminationStatus == 0
		fail "couldn't generate strings for #{currentLocation}"
	end
	
	print "\n"
end

finish
